@@ -22,6 +22,7 @@
 #include <wchar.h>
 #define HAVE_STDARG_H
 #include <86box/86box.h>
+#include "cpu.h"
 #include <86box/timer.h>
 #include <86box/device.h>
 #include <86box/io.h>
@@ -1599,8 +1600,7 @@ ali1543_close(void *priv)
 static void *
 ali1543_init(const device_t *info)
 {
-    ali1543_t *dev = (ali1543_t *) malloc(sizeof(ali1543_t));
-    memset(dev, 0, sizeof(ali1543_t));
+    ali1543_t *dev = (ali1543_t *) calloc(1, sizeof(ali1543_t));
 
     /* Device 02: M1533 Southbridge */
     pci_add_card(PCI_ADD_SOUTHBRIDGE, ali1533_read, ali1533_write, dev, &dev->pci_slot);
@@ -1673,7 +1673,7 @@ const device_t ali1543_device = {
     .init  = ali1543_init,
     .close = ali1543_close,
     .reset = ali1543_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1687,7 +1687,7 @@ const device_t ali1543c_device = {
     .init          = ali1543_init,
     .close         = ali1543_close,
     .reset         = ali1543_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
