@@ -594,8 +594,7 @@ headland_init(const device_t *info)
     headland_t *dev;
     int         ht386 = 0;
 
-    dev = (headland_t *) malloc(sizeof(headland_t));
-    memset(dev, 0x00, sizeof(headland_t));
+    dev = (headland_t *) calloc(1, sizeof(headland_t));
 
     dev->has_cri   = (info->local & HEADLAND_HAS_CRI);
     dev->has_sleep = (info->local & HEADLAND_HAS_SLEEP);
@@ -699,7 +698,7 @@ const device_t headland_gc10x_device = {
     .init          = headland_init,
     .close         = headland_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -713,7 +712,7 @@ const device_t headland_gc113_device = {
     .init          = headland_init,
     .close         = headland_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -727,7 +726,7 @@ const device_t headland_ht18a_device = {
     .init          = headland_init,
     .close         = headland_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -741,7 +740,7 @@ const device_t headland_ht18b_device = {
     .init          = headland_init,
     .close         = headland_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -755,7 +754,7 @@ const device_t headland_ht18c_device = {
     .init          = headland_init,
     .close         = headland_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -769,7 +768,7 @@ const device_t headland_ht21c_d_device = {
     .init          = headland_init,
     .close         = headland_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -783,7 +782,7 @@ const device_t headland_ht21e_device = {
     .init          = headland_init,
     .close         = headland_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
