@@ -47,6 +47,7 @@
 #include <86box/hdc_ide.h>
 #include <86box/hdc_ide_sff8038i.h>
 #include <86box/usb.h>
+#include <86box/lpt.h>
 #include <86box/machine.h>
 #include <86box/smbus.h>
 #include <86box/chipset.h>
@@ -59,7 +60,6 @@ typedef struct piix_io_trap_t {
 } piix_io_trap_t;
 
 typedef struct _piix_ {
-    uint8_t        cur_readout_reg;
     uint8_t        rev;
     uint8_t        type;
     uint8_t        func_shift;
@@ -67,7 +67,6 @@ typedef struct _piix_ {
     uint8_t        pci_slot;
     uint8_t        no_mirq0;
     uint8_t        regs[4][256];
-    uint8_t        readout_regs[256];
     uint16_t       func0_id;
     uint16_t       nvr_io_base;
     uint16_t       acpi_io_base;
@@ -157,6 +156,7 @@ piix_ide_handlers(piix_t *dev, int bus)
     uint16_t side;
 
     if (bus & 0x01) {
+        piix_log("Disabling primary IDE...\n");
         ide_pri_disable();
 
         if (dev->type == 5) {
@@ -172,11 +172,14 @@ piix_ide_handlers(piix_t *dev, int bus)
             ide_set_side(0, side);
         }
 
-        if ((dev->regs[1][0x04] & 0x01) && (dev->regs[1][0x41] & 0x80))
+        if ((dev->regs[1][0x04] & 0x01) && (dev->regs[1][0x41] & 0x80)) {
+            piix_log("Enabling primary IDE...\n");
             ide_pri_enable();
+        }
     }
 
     if (bus & 0x02) {
+        piix_log("Disabling secondary IDE...\n");
         ide_sec_disable();
 
         if (dev->type == 5) {
@@ -192,8 +195,10 @@ piix_ide_handlers(piix_t *dev, int bus)
             ide_set_side(1, side);
         }
 
-        if ((dev->regs[1][0x04] & 0x01) && (dev->regs[1][0x43] & 0x80))
+        if ((dev->regs[1][0x04] & 0x01) && (dev->regs[1][0x43] & 0x80)) {
+            piix_log("Enabling secondary IDE...\n");
             ide_sec_enable();
+        }
     }
 }
 
@@ -467,6 +472,13 @@ piix_write(int func, int addr, uint8_t val, void *priv)
     uint8_t *fregs;
     uint16_t base;
 
+    /* Dell OptiPlex Gn+ shows that register 02:FF is aliased in 01:FF. */
+    if ((dev->type == 4) && (func == 1) && (addr == 0xff))
+        func = 2;
+
+    if ((func == 1) || (addr == 0xf8) || (addr == 0xf9))
+        piix_log("[W] %02X:%02X = %02X\n", func, addr, val);
+
     /* Return on unsupported function. */
     if (dev->max_func > 0) {
         if (func > dev->max_func)
@@ -599,14 +611,21 @@ piix_write(int func, int addr, uint8_t val, void *priv)
                         pci_set_mirq_routing(PCI_MIRQ0 + (addr & 0x01), val & 0xf);
                     if (dev->type == 3) {
                         if (val & 0x20)
-                            sff_set_irq_mode(dev->bm[1], IRQ_MODE_MIRQ_0);
-                        else
                             sff_set_irq_mode(dev->bm[1], IRQ_MODE_LEGACY);
+                        else
+                            sff_set_irq_mode(dev->bm[1], IRQ_MODE_MIRQ_0);
                     }
                     piix_log("MIRQ%i is %s\n", addr & 0x01, (val & 0x20) ? "disabled" : "enabled");
                 }
                 break;
             case 0x76:
+                if (dev->type > 1)
+                    fregs[addr] = val & 0x87;
+                else if (dev->type <= 4)
+                    fregs[addr] = val & 0x8f;
+                if ((dev->type == 1) && machine_has_jumpered_ecp_dma(machine, MACHINE_DMA_USE_MBDMA))
+                    lpt1_dma(((val & 0x08) || ((val & 0x07) == 0x04)) ? 0xff : (val & 0x07));
+                break;
             case 0x77:
                 if (dev->type > 1)
                     fregs[addr] = val & 0x87;
@@ -738,6 +757,8 @@ piix_write(int func, int addr, uint8_t val, void *priv)
                     fregs[addr] = val;
                 break;
             case 0xb0:
+                if (val & 0x10)
+                    warning("Write %02X to B0\n", val);
                 if (dev->type == 4)
                     fregs[addr] = (fregs[addr] & 0x8c) | (val & 0x73);
                 else if (dev->type == 5)
@@ -747,6 +768,8 @@ piix_write(int func, int addr, uint8_t val, void *priv)
                     alt_access = !!(val & 0x20);
                 break;
             case 0xb1:
+                if (val & 0x18)
+                    warning("Write %02X to B1\n", val);
                 if (dev->type > 3)
                     fregs[addr] = val & 0xdf;
                 break;
@@ -925,6 +948,12 @@ piix_write(int func, int addr, uint8_t val, void *priv)
                 if (dev->type > 4)
                     fregs[addr] = val;
                 break;
+            case 0xf8:
+            case 0xf9:
+                /* Undocumented! */
+                if (dev->type == 4)
+                    fregs[addr] = val;
+                break;
             default:
                 break;
         }
@@ -1171,6 +1200,10 @@ piix_read(int func, int addr, void *priv)
     uint8_t        ret = 0xff;
     const uint8_t *fregs;
 
+    /* Dell OptiPlex Gn+ shows that register 02:FF is aliased in 01:FF. */
+    if ((dev->type == 4) && (func == 1) && (addr == 0xff))
+        func = 2;
+
     if ((dev->type == 3) && (func == 2) && (dev->max_func == 1) && (addr >= 0x40))
         ret = 0x00;
 
@@ -1185,31 +1218,6 @@ piix_read(int func, int addr, void *priv)
     return ret;
 }
 
-static void
-board_write(uint16_t port, uint8_t val, void *priv)
-{
-    piix_t *dev = (piix_t *) priv;
-
-    if (port == 0x00e0)
-        dev->cur_readout_reg = val;
-    else if (port == 0x00e1)
-        dev->readout_regs[dev->cur_readout_reg] = val;
-}
-
-static uint8_t
-board_read(uint16_t port, void *priv)
-{
-    const piix_t *dev = (piix_t *) priv;
-    uint8_t       ret = 0x64;
-
-    if (port == 0x00e0)
-        ret = dev->cur_readout_reg;
-    else if (port == 0x00e1)
-        ret = dev->readout_regs[dev->cur_readout_reg];
-
-    return ret;
-}
-
 static void
 piix_reset_hard(piix_t *dev)
 {
@@ -1226,7 +1234,7 @@ piix_reset_hard(piix_t *dev)
 
         sff_set_slot(dev->bm[1], dev->pci_slot);
         sff_set_irq_pin(dev->bm[1], PCI_INTA);
-        sff_set_irq_line(dev->bm[1], 14);
+        sff_set_irq_line(dev->bm[1], 15);
         sff_set_irq_mode(dev->bm[1], IRQ_MODE_LEGACY);
     }
 
@@ -1342,6 +1350,10 @@ piix_reset_hard(piix_t *dev)
         fregs[0x45] = 0x55;
         fregs[0x46] = 0x01;
     }
+    if (dev->type == 4) {
+        fregs[0xf8] = 0x30;
+        fregs[0xf9] = 0x0f;
+    }
     if ((dev->type == 1) && (dev->rev == 2))
         dev->max_func = 0; /* It starts with IDE disabled, then enables it. */
     else
@@ -1538,8 +1550,7 @@ piix_speed_changed(void *priv)
 static void *
 piix_init(const device_t *info)
 {
-    piix_t *dev = (piix_t *) malloc(sizeof(piix_t));
-    memset(dev, 0, sizeof(piix_t));
+    piix_t *dev = (piix_t *) calloc(1, sizeof(piix_t));
 
     dev->type = info->local & 0x0f;
     /* If (dev->type == 4) and (dev->rev & 0x08), then this is PIIX4E. */
@@ -1618,47 +1629,16 @@ piix_init(const device_t *info)
     else
         cpu_set_isa_pci_div(3);
 
-    dma_alias_set();
+    if (dev->type > 1)
+        dma_alias_set();
+    else
+        dma_alias_set_piix();
 
     if (dev->type < 4)
         pci_enable_mirq(0);
     if (dev->type < 3)
         pci_enable_mirq(1);
 
-    dev->readout_regs[0] = 0xff;
-    dev->readout_regs[1] = 0x40;
-    dev->readout_regs[2] = 0xff;
-
-    /* Port E1 register 01 (TODO: Find how multipliers > 3.0 are defined):
-
-        Bit 6: 1 = can boot, 0 = no;
-        Bit 7, 1 = multiplier (00 = 2.5, 01 = 2.0, 10 = 3.0, 11 = 1.5);
-        Bit 5, 4 = bus speed (00 = 50 MHz, 01 = 66 MHz, 10 = 60 MHz, 11 = ????):
-        Bit 7, 5, 4, 1: 0000 = 125 MHz, 0010 = 166 MHz, 0100 = 150 MHz, 0110 = ??? MHz;
-                        0001 = 100 MHz, 0011 = 133 MHz, 0101 = 120 MHz, 0111 = ??? MHz;
-                        1000 = 150 MHz, 1010 = 200 MHz, 1100 = 180 MHz, 1110 = ??? MHz;
-                        1001 =  75 MHz, 1011 = 100 MHz, 1101 =  90 MHz, 1111 = ??? MHz */
-
-    if (cpu_busspeed <= 40000000)
-        dev->readout_regs[1] |= 0x30;
-    else if ((cpu_busspeed > 40000000) && (cpu_busspeed <= 50000000))
-        dev->readout_regs[1] |= 0x00;
-    else if ((cpu_busspeed > 50000000) && (cpu_busspeed <= 60000000))
-        dev->readout_regs[1] |= 0x20;
-    else if (cpu_busspeed > 60000000)
-        dev->readout_regs[1] |= 0x10;
-
-    if (cpu_dmulti <= 1.5)
-        dev->readout_regs[1] |= 0x82;
-    else if ((cpu_dmulti > 1.5) && (cpu_dmulti <= 2.0))
-        dev->readout_regs[1] |= 0x02;
-    else if ((cpu_dmulti > 2.0) && (cpu_dmulti <= 2.5))
-        dev->readout_regs[1] |= 0x00;
-    else if (cpu_dmulti > 2.5)
-        dev->readout_regs[1] |= 0x80;
-
-    io_sethandler(0x00e0, 0x0002, board_read, NULL, NULL, board_write, NULL, NULL, dev);
-
 #if 0
     device_add(&i8254_sec_device);
 #endif
@@ -1674,7 +1654,7 @@ const device_t piix_device = {
     .init          = piix_init,
     .close         = piix_close,
     .reset         = piix_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = piix_speed_changed,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1688,7 +1668,7 @@ const device_t piix_no_mirq_device = {
     .init          = piix_init,
     .close         = piix_close,
     .reset         = piix_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = piix_speed_changed,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1702,7 +1682,7 @@ const device_t piix_rev02_device = {
     .init          = piix_init,
     .close         = piix_close,
     .reset         = piix_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = piix_speed_changed,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1716,7 +1696,7 @@ const device_t piix3_device = {
     .init          = piix_init,
     .close         = piix_close,
     .reset         = piix_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = piix_speed_changed,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1730,7 +1710,7 @@ const device_t piix3_ioapic_device = {
     .init          = piix_init,
     .close         = piix_close,
     .reset         = piix_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = piix_speed_changed,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1744,7 +1724,7 @@ const device_t piix4_device = {
     .init          = piix_init,
     .close         = piix_close,
     .reset         = piix_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = piix_speed_changed,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1758,7 +1738,7 @@ const device_t piix4e_device = {
     .init          = piix_init,
     .close         = piix_close,
     .reset         = piix_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = piix_speed_changed,
     .force_redraw  = NULL,
     .config        = NULL
@@ -1772,7 +1752,7 @@ const device_t slc90e66_device = {
     .init          = piix_init,
     .close         = piix_close,
     .reset         = piix_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = piix_speed_changed,
     .force_redraw  = NULL,
     .config        = NULL
