@@ -21,11 +21,13 @@ extern int cdrom_interface_current;
 
 extern void cdrom_interface_reset(void);
 
-extern const char     *cdrom_interface_get_internal_name(int cdinterface);
-extern int             cdrom_interface_get_from_internal_name(char *s);
-extern int             cdrom_interface_has_config(int cdinterface);
-extern const device_t *cdrom_interface_get_device(int cdinterface);
-extern int             cdrom_interface_get_flags(int cdinterface);
-extern int             cdrom_interface_available(int cdinterface);
+const  char           *cdrom_interface_get_internal_name(const int cdinterface);
+extern int             cdrom_interface_get_from_internal_name(const char *s);
+#ifdef EMU_DEVICE_H
+extern const device_t *cdrom_interface_get_device(const int cdinterface);
+#endif
+extern int             cdrom_interface_has_config(const int cdinterface);
+extern int             cdrom_interface_get_flags(const int cdinterface);
+extern int             cdrom_interface_available(const int cdinterface);
 
 #endif /*EMU_CDROM_INTERFACE_H*/
