@@ -412,7 +412,7 @@ extern const device_t cga_pravetz_device;
 /* Compaq CGA */
 extern const device_t compaq_cga_device;
 extern const device_t compaq_cga_2_device;
-extern const device_t compaq_plasma_device; 
+extern const device_t compaq_plasma_device;
 
 /* Olivetti OGC */
 extern const device_t ogc_device;
@@ -490,6 +490,7 @@ extern const device_t oti067_device;
 extern const device_t oti067_acer386_device;
 extern const device_t oti067_ama932j_device;
 extern const device_t oti077_acer100t_device;
+extern const device_t oti077_pcs44c_device;
 extern const device_t oti077_device;
 
 /* Paradise/WD (S)VGA */
@@ -512,8 +513,11 @@ extern const device_t realtek_rtg3106_device;
 extern const device_t s3_orchid_86c911_isa_device;
 extern const device_t s3_diamond_stealth_vram_isa_device;
 extern const device_t s3_ami_86c924_isa_device;
+extern const device_t s3_elsa_winner1000_86c928_vlb_device;
+extern const device_t s3_elsa_winner2000_86c928_isa_device;
 extern const device_t s3_metheus_86c928_isa_device;
 extern const device_t s3_metheus_86c928_vlb_device;
+extern const device_t s3_elsa_winner1000_86c928_pci_device;
 extern const device_t s3_spea_mercury_lite_86c928_pci_device;
 extern const device_t s3_spea_mirage_86c801_isa_device;
 extern const device_t s3_winner1000_805_isa_device;
@@ -634,9 +638,9 @@ extern const device_t wy700_device;
 extern const device_t v6355d_device;
 
 /* Tandy */
-extern const device_t tandy_1000_video_device; 
-extern const device_t tandy_1000hx_video_device; 
-extern const device_t tandy_1000sl_video_device; 
+extern const device_t tandy_1000_video_device;
+extern const device_t tandy_1000hx_video_device;
+extern const device_t tandy_1000sl_video_device;
 
 #endif
 
