@@ -273,8 +273,10 @@ cs423x_write(uint16_t addr, uint8_t val, void *priv)
             }
             switch (dev->regs[3] & 0x0f) {
                 case 0: /* WSS Master Control */
-                    if ((dev->type < CRYSTAL_CS4235) && (val & 0x80))
+                    if ((dev->type < CRYSTAL_CS4235) && (val & 0x80)) {
                         ad1848_init(&dev->ad1848, dev->ad1848_type);
+                        ad1848_set_cd_audio_channel(&dev->ad1848, AD1848_AUX2);
+                    }
                     val = 0x00;
                     break;
 
@@ -865,6 +867,7 @@ cs423x_reset(void *priv)
 
     /* Reset WSS codec. */
     ad1848_init(&dev->ad1848, dev->ad1848_type);
+    ad1848_set_cd_audio_channel(&dev->ad1848, AD1848_AUX2);
 
     /* Reset PnP resource data, state and logical devices. */
     dev->pnp_enable = 1;
