@@ -33,8 +33,12 @@
 #include <86box/rom.h>
 #include <86box/device.h>
 #include <86box/video.h>
+#include <86box/vid_8514a.h>
+#include <86box/vid_xga.h>
 #include <86box/vid_svga.h>
 #include <86box/vid_svga_render.h>
+#include <86box/vid_ati_eeprom.h>
+#include <86box/vid_ati_mach8.h>
 #include <86box/plat_fallthrough.h>
 #include <86box/plat_unused.h>
 
@@ -424,9 +428,8 @@ ht216_out(uint16_t addr, uint8_t val, void *priv)
                     svga->banked_mask = 0xffff;
             }
 
-            if (svga->gdcaddr <= 8) {
+            if (svga->gdcaddr <= 8)
                 svga->fast = (svga->gdcreg[8] == 0xff && !(svga->gdcreg[3] & 0x18) && !svga->gdcreg[1]) && svga->chain4 && svga->packed_chain4;
-            }
             break;
 
         case 0x3D4:
@@ -445,7 +448,7 @@ ht216_out(uint16_t addr, uint8_t val, void *priv)
                 if (svga->crtcreg < 0xe || svga->crtcreg > 0x10) {
                     if ((svga->crtcreg == 0xc) || (svga->crtcreg == 0xd)) {
                         svga->fullchange = 3;
-                        svga->ma_latch   = ((svga->crtc[0xc] << 8) | svga->crtc[0xd]) + ((svga->crtc[8] & 0x60) >> 5);
+                        svga->memaddr_latch   = ((svga->crtc[0xc] << 8) | svga->crtc[0xd]) + ((svga->crtc[8] & 0x60) >> 5);
                     } else {
                         svga->fullchange = changeframecount;
                         svga_recalctimings(svga);
@@ -624,7 +627,9 @@ ht216_remap(ht216_t *ht216)
 void
 ht216_recalctimings(svga_t *svga)
 {
-    ht216_t *ht216        = (ht216_t *) svga->priv;
+    ht216_t   *ht216      = (ht216_t *) svga->priv;
+    ibm8514_t *dev        = (ibm8514_t *) svga->dev8514;
+    mach_t    *mach       = (mach_t *) svga->ext8514;
     int      high_res_256 = 0;
 
 
@@ -655,10 +660,10 @@ ht216_recalctimings(svga_t *svga)
             break;
     }
 
-    svga->ma_latch |= ((ht216->ht_regs[0xf6] & 0x30) << 12);
+    svga->memaddr_latch |= ((ht216->ht_regs[0xf6] & 0x30) << 12);
 
     if (ht216->ht_regs[0xf6] & 0x80)
-        svga->ma_latch = ((ht216->ht_regs[0xf6] & 0x30) << 12);
+        svga->memaddr_latch = ((ht216->ht_regs[0xf6] & 0x30) << 12);
 
     svga->interlace = ht216->ht_regs[0xe0] & 0x01;
 
@@ -671,10 +676,16 @@ ht216_recalctimings(svga_t *svga)
 
     if (!svga->scrblank && svga->attr_palette_enable) {
         if (!(svga->gdcreg[6] & 1) && !(svga->attrregs[0x10] & 1)) { /*Text mode*/
-            if (svga->seqregs[1] & 8) /*40 column*/ {
+            if (svga->seqregs[1] & 8) /*40 column*/
                 svga->render = svga_render_text_40;
-            } else {
+            else
                 svga->render = svga_render_text_80;
+
+            if (ibm8514_active && (svga->dev8514 != NULL)) {
+                if (svga->ext8514 != NULL) {
+                    if (!(dev->accel.advfunc_cntl & 0x01) && !(mach->accel.clock_sel & 0x01)) /*FIXME: Possibly a BIOS bug within the V7 chips when it's used with a 8514/A card?*/
+                        dev->on &= ~0x01;
+                }
             }
         } else {
             if (svga->crtc[0x17] == 0xeb) {
@@ -687,12 +698,16 @@ ht216_recalctimings(svga_t *svga)
                 if (((ht216->ht_regs[0xc8] & HT_REG_C8_E256) || (svga->gdcreg[5] & 0x40)) && (!svga->lowres || (ht216->ht_regs[0xf6] & 0x80))) {
                     if (high_res_256) {
                         svga->hdisp >>= 1;
+                        svga->dots_per_clock >>= 1;
+                        svga->clock /= 2;
                         ht216->adjust_cursor = 1;
                     }
                     svga->render = svga_render_8bpp_highres;
                 } else if (svga->lowres) {
                     if (high_res_256) {
                         svga->hdisp >>= 1;
+                        svga->dots_per_clock >>= 1;
+                        svga->clock /= 2;
                         ht216->adjust_cursor = 1;
                         svga->render         = svga_render_8bpp_highres;
                     } else {
@@ -715,6 +730,8 @@ ht216_recalctimings(svga_t *svga)
             } else if (svga->bpp == 15) {
                 svga->rowoffset <<= 1;
                 svga->hdisp >>= 1;
+                svga->dots_per_clock >>= 1;
+                svga->clock /= 2;
                 if ((svga->crtc[0x17] & 0x60) == 0x20) /*Would result in a garbled screen with trailing cursor glitches*/
                     svga->crtc[0x17] |= 0x40;
                 svga->render = svga_render_15bpp_highres;
@@ -1217,6 +1234,8 @@ ht216_write(uint32_t addr, uint8_t val, void *priv)
     svga_t  *svga      = &ht216->svga;
     uint32_t prev_addr = addr;
 
+    xga_write_test(addr, val, svga);
+
     addr &= svga->banked_mask;
     addr = (addr & 0x7fff) + ht216->write_banks[(addr >> 15) & 1];
 
@@ -1238,6 +1257,9 @@ ht216_writew(uint32_t addr, uint16_t val, void *priv)
     svga_t  *svga      = &ht216->svga;
     uint32_t prev_addr = addr;
 
+    xga_write_test(addr, val, svga);
+    xga_write_test(addr + 1, val >> 8, svga);
+
     addr &= svga->banked_mask;
     addr = (addr & 0x7fff) + ht216->write_banks[(addr >> 15) & 1];
 
@@ -1261,6 +1283,11 @@ ht216_writel(uint32_t addr, uint32_t val, void *priv)
     svga_t  *svga      = &ht216->svga;
     uint32_t prev_addr = addr;
 
+    xga_write_test(addr, val, svga);
+    xga_write_test(addr + 1, val >> 8, svga);
+    xga_write_test(addr + 2, val >> 16, svga);
+    xga_write_test(addr + 3, val >> 24, svga);
+
     addr &= svga->banked_mask;
     addr = (addr & 0x7fff) + ht216->write_banks[(addr >> 15) & 1];
 
@@ -1422,9 +1449,11 @@ static uint8_t
 ht216_read(uint32_t addr, void *priv)
 {
     ht216_t      *ht216     = (ht216_t *) priv;
-    const svga_t *svga      = &ht216->svga;
+    svga_t       *svga      = &ht216->svga;
     uint32_t      prev_addr = addr;
 
+    (void) xga_read_test(addr, svga);
+
     addr &= svga->banked_mask;
     addr = (addr & 0x7fff) + ht216->read_banks[(addr >> 15) & 1];
 
@@ -1563,10 +1592,17 @@ ht216_init(const device_t *info, uint32_t mem_size, int has_rom)
     if (has_rom == 4)
         svga->ramdac = device_add(&sc11484_nors2_ramdac_device);
 
+    svga->read = ht216_read;
+    svga->readw = NULL;
+    svga->readl = NULL;
+    svga->write = ht216_write;
+    svga->writew = ht216_writew;
     if ((info->flags & DEVICE_VLB) || (info->flags & DEVICE_MCA)) {
+        svga->writel = ht216_writel;
         mem_mapping_set_handler(&svga->mapping, ht216_read, NULL, NULL, ht216_write, ht216_writew, ht216_writel);
         mem_mapping_add(&ht216->linear_mapping, 0, 0, ht216_read_linear, NULL, NULL, ht216_write_linear, ht216_writew_linear, ht216_writel_linear, NULL, MEM_MAPPING_EXTERNAL, svga);
     } else {
+        svga->writel = NULL;
         mem_mapping_set_handler(&svga->mapping, ht216_read, NULL, NULL, ht216_write, ht216_writew, NULL);
         mem_mapping_add(&ht216->linear_mapping, 0, 0, ht216_read_linear, NULL, NULL, ht216_write_linear, ht216_writew_linear, NULL, NULL, MEM_MAPPING_EXTERNAL, svga);
     }
@@ -1574,7 +1610,7 @@ ht216_init(const device_t *info, uint32_t mem_size, int has_rom)
     mem_mapping_disable(&ht216->linear_mapping);
 
     ht216->id     = info->local;
-    ht216->isabus = (info->flags & DEVICE_ISA);
+    ht216->isabus = (info->flags & DEVICE_ISA) || (info->flags & DEVICE_ISA16);
     ht216->mca    = (info->flags & DEVICE_MCA);
 
     io_sethandler(0x03c0, 0x0020, ht216_in, NULL, NULL, ht216_out, NULL, NULL, ht216);
@@ -1688,52 +1724,45 @@ ht216_force_redraw(void *priv)
     ht216->svga.fullchange = changeframecount;
 }
 
+// clang-format off
 static const device_config_t v7_vga_1024i_config[] = {
-    { .name        = "memory",
-     .description = "Memory size",
-     .type        = CONFIG_SELECTION,
-     .default_int = 512,
-     .selection   = {
-          { .description = "256 kB",
-              .value       = 256 },
-          { .description = "512 kB",
-              .value       = 512 },
-          { .description = "" } } },
-    { .type = CONFIG_END }
+    {
+        .name           = "memory",
+        .description    = "Memory size",
+        .type           = CONFIG_SELECTION,
+        .default_string = NULL,
+        .default_int    = 512,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = {
+            { .description = "256 KB", .value = 256 },
+            { .description = "512 KB", .value = 512 },
+            { .description = ""                     }
+        },
+        .bios           = { { 0 } }
+    },
+    { .name = "", .description = "", .type = CONFIG_END }
 };
 
-// clang-format off
 static const device_config_t ht216_32_standalone_config[] = {
     {
-        .name = "monitor_type",
-        .description = "Monitor type",
-        .type = CONFIG_SELECTION,
-        .default_int = 0x18,
-        .selection = {
-            {
-                .description = "Mono Interlaced",
-                .value = 0x00
-            },
-            {
-                .description = "Mono Non-Interlaced",
-                .value = 0x08
-            },
-            {
-                .description = "Color Interlaced",
-                .value = 0x10
-            },
-            {
-                .description = "Color Non-Interlaced",
-                .value = 0x18
-            },
-            {
-                .description = ""
-            }
-        }
+        .name           = "monitor_type",
+        .description    = "Monitor type",
+        .type           = CONFIG_SELECTION,
+        .default_string = NULL,
+        .default_int    = 0x18,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = {
+            { .description = "Mono Interlaced",      .value = 0x00 },
+            { .description = "Mono Non-Interlaced",  .value = 0x08 },
+            { .description = "Color Interlaced",     .value = 0x10 },
+            { .description = "Color Non-Interlaced", .value = 0x18 },
+            { .description = ""                                    }
+        },
+        .bios           = { { 0 } }
     },
-    {
-        .type = CONFIG_END
-    }
+    { .name = "", .description = "", .type = CONFIG_END }
 };
 // clang-format on
 
@@ -1745,7 +1774,7 @@ const device_t g2_gc205_device = {
     .init          = g2_gc205_init,
     .close         = ht216_close,
     .reset         = NULL,
-    { .available = g2_gc205_available },
+    .available     = g2_gc205_available,
     .speed_changed = ht216_speed_changed,
     .force_redraw  = ht216_force_redraw,
     .config        = NULL
@@ -1759,7 +1788,7 @@ const device_t v7_vga_1024i_device = {
     .init          = v7_vga_1024i_init,
     .close         = ht216_close,
     .reset         = NULL,
-    { .available = v7_vga_1024i_available },
+    .available     = v7_vga_1024i_available,
     .speed_changed = ht216_speed_changed,
     .force_redraw  = ht216_force_redraw,
     .config        = v7_vga_1024i_config
@@ -1773,7 +1802,7 @@ const device_t ht216_32_pb410a_device = {
     .init          = ht216_pb410a_init,
     .close         = ht216_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = ht216_speed_changed,
     .force_redraw  = ht216_force_redraw,
     .config        = NULL
@@ -1787,7 +1816,7 @@ const device_t ht216_32_standalone_device = {
     .init          = ht216_standalone_init,
     .close         = ht216_close,
     .reset         = NULL,
-    { .available = ht216_standalone_available },
+    .available     = ht216_standalone_available,
     .speed_changed = ht216_speed_changed,
     .force_redraw  = ht216_force_redraw,
     .config        = ht216_32_standalone_config
@@ -1796,12 +1825,12 @@ const device_t ht216_32_standalone_device = {
 const device_t radius_svga_multiview_isa_device = {
     .name          = "Radius SVGA Multiview ISA (HT209)",
     .internal_name = "radius_isa",
-    .flags         = DEVICE_ISA | DEVICE_AT,
+    .flags         = DEVICE_ISA16,
     .local         = 0x7152, /*HT209*/
     .init          = radius_svga_multiview_init,
     .close         = ht216_close,
     .reset         = NULL,
-    { .available = radius_svga_multiview_available },
+    .available     = radius_svga_multiview_available,
     .speed_changed = ht216_speed_changed,
     .force_redraw  = ht216_force_redraw,
     .config        = NULL
@@ -1815,7 +1844,7 @@ const device_t radius_svga_multiview_mca_device = {
     .init          = radius_svga_multiview_init,
     .close         = ht216_close,
     .reset         = NULL,
-    { .available = radius_svga_multiview_available },
+    .available     = radius_svga_multiview_available,
     .speed_changed = ht216_speed_changed,
     .force_redraw  = ht216_force_redraw,
     .config        = NULL
