@@ -30,9 +30,18 @@
 #include <86box/rom.h>
 #include <86box/device.h>
 #include <86box/video.h>
+#include <86box/vid_xga.h>
 #include <86box/vid_svga.h>
 #include <86box/vid_svga_render.h>
 
+#define VAR_BYTE_MODE      (0 << 0)
+#define VAR_WORD_MODE_MA13 (1 << 0)
+#define VAR_WORD_MODE_MA15 (2 << 0)
+#define VAR_DWORD_MODE     (3 << 0)
+#define VAR_MODE_MASK      (3 << 0)
+#define VAR_ROW0_MA13      (1 << 2)
+#define VAR_ROW1_MA14      (1 << 3)
+
 typedef struct paradise_t {
     svga_t svga;
 
@@ -68,7 +77,7 @@ typedef struct paradise_t {
 } paradise_t;
 
 static video_timings_t timing_paradise_pvga1a = { .type = VIDEO_ISA, .write_b = 6, .write_w = 8, .write_l = 16, .read_b = 6, .read_w = 8, .read_l = 16 };
-static video_timings_t timing_paradise_wd90c  = { .type = VIDEO_ISA, .write_b = 3, .write_w = 3, .write_l = 6, .read_b = 5, .read_w = 5, .read_l = 10 };
+static video_timings_t timing_paradise_wd90c  = { .type = VIDEO_ISA, .write_b = 3, .write_w = 3, .write_l =  6, .read_b = 5, .read_w = 5, .read_l = 10 };
 
 void paradise_remap(paradise_t *paradise);
 
@@ -77,6 +86,7 @@ paradise_in(uint16_t addr, void *priv)
 {
     paradise_t *paradise = (paradise_t *) priv;
     svga_t     *svga     = &paradise->svga;
+    uint8_t     max_sr   = (paradise->type >= WD90C30) ? 0x15 : 0x12;
 
     if (((addr & 0xfff0) == 0x3d0 || (addr & 0xfff0) == 0x3b0) && !(svga->miscout & 1))
         addr ^= 0x60;
@@ -86,7 +96,7 @@ paradise_in(uint16_t addr, void *priv)
             if (svga->seqaddr > 7) {
                 if (paradise->type < WD90C11 || svga->seqregs[6] != 0x48)
                     return 0xff;
-                if (svga->seqaddr > 0x12)
+                if (svga->seqaddr > max_sr)
                     return 0xff;
                 return svga->seqregs[svga->seqaddr & 0x1f];
             }
@@ -134,6 +144,7 @@ paradise_out(uint16_t addr, uint8_t val, void *priv)
 {
     paradise_t *paradise = (paradise_t *) priv;
     svga_t     *svga     = &paradise->svga;
+    xga_t      *xga      = (xga_t *) svga->xga;
     uint8_t     old;
 
     if (((addr & 0xfff0) == 0x3d0 || (addr & 0xfff0) == 0x3b0) && !(svga->miscout & 1))
@@ -179,6 +190,10 @@ paradise_out(uint16_t addr, uint8_t val, void *priv)
                             case 0x4: /*64k at A0000*/
                                 mem_mapping_set_addr(&svga->mapping, 0xa0000, 0x10000);
                                 svga->banked_mask = 0xffff;
+                                if (xga_active && (svga->xga != NULL)) {
+                                    xga->on = 0;
+                                    mem_mapping_set_handler(&svga->mapping, svga->read, svga->readw, svga->readl, svga->write, svga->writew, svga->writel);
+                                }
                                 break;
                             case 0x8: /*32k at B0000*/
                                 mem_mapping_set_addr(&svga->mapping, 0xb0000, 0x08000);
@@ -204,6 +219,28 @@ paradise_out(uint16_t addr, uint8_t val, void *priv)
                     return;
                 case 0x0b:
                     svga->gdcreg[0x0b] = val;
+                    svga->gdcreg[0x0b] &= ~0xc0;
+                    if (paradise->memory == 1024)
+                        svga->gdcreg[0x0b] |= 0xc0;
+                    else if (paradise->memory == 512)
+                        svga->gdcreg[0x0b] |= 0x80;
+                    else
+                        svga->gdcreg[0x0b] |= 0x40;
+
+                    if (svga->crtc[0x2f] & 0x02)
+                        svga->decode_mask = 0x3ffff;
+                    else  switch (svga->gdcreg[0x0b] & 0xc0) {
+                        case 0x00: case 0x40:
+                            svga->decode_mask = 0x3ffff;
+                            break;
+                        case 0x80:
+                            svga->decode_mask = 0x7ffff;
+                            break;
+                        case 0xc0:
+                            svga->decode_mask = 0xfffff;
+                            break;
+                    }
+
                     paradise_remap(paradise);
                     return;
                 case 0x0e:
@@ -235,9 +272,24 @@ paradise_out(uint16_t addr, uint8_t val, void *priv)
 
             if (old != val) {
                 if (svga->crtcreg < 0xe || svga->crtcreg > 0x10) {
+                    if (svga->crtcreg == 0x2f) {
+                        if (svga->crtc[0x2f] & 0x02)
+                            svga->decode_mask = 0x3ffff;
+                        else  switch (svga->gdcreg[0x0b] & 0xc0) {
+                            case 0x00: case 0x40:
+                                svga->decode_mask = 0x3ffff;
+                                break;
+                            case 0x80:
+                                svga->decode_mask = 0x7ffff;
+                                break;
+                            case 0xc0:
+                                svga->decode_mask = 0xfffff;
+                                break;
+                        }
+                    }
                     if ((svga->crtcreg == 0xc) || (svga->crtcreg == 0xd)) {
                         svga->fullchange = 3;
-                        svga->ma_latch   = ((svga->crtc[0xc] << 8) | svga->crtc[0xd]) + ((svga->crtc[8] & 0x60) >> 5);
+                        svga->memaddr_latch   = ((svga->crtc[0xc] << 8) | svga->crtc[0xd]) + ((svga->crtc[8] & 0x60) >> 5);
                     } else {
                         svga->fullchange = changeframecount;
                         svga_recalctimings(svga);
@@ -259,29 +311,92 @@ paradise_remap(paradise_t *paradise)
     svga_t *svga    = &paradise->svga;
 
     if (svga->seqregs[0x11] & 0x80) {
-        paradise->read_bank[0] = paradise->read_bank[2] = svga->gdcreg[9] << 12;
-        paradise->read_bank[1] = paradise->read_bank[3] = (svga->gdcreg[9] << 12) + ((svga->gdcreg[6] & 0x08) ? 0 : 0x8000);
-        paradise->write_bank[0] = paradise->write_bank[2] = svga->gdcreg[0x0a] << 12;
-        paradise->write_bank[1] = paradise->write_bank[3] = (svga->gdcreg[0x0a] << 12) + ((svga->gdcreg[6] & 0x08) ? 0 : 0x8000);
-    } else if (svga->gdcreg[0x0b] & 0x08) {
-        if (svga->gdcreg[6] & 0x0c) {
-            paradise->read_bank[0] = paradise->read_bank[2] = svga->gdcreg[0x0a] << 12;
-            paradise->write_bank[0] = paradise->write_bank[2] = svga->gdcreg[0x0a] << 12;
-            paradise->read_bank[1] = paradise->read_bank[3] = (svga->gdcreg[9] << 12) + ((svga->gdcreg[6] & 0x08) ? 0 : 0x8000);
-            paradise->write_bank[1] = paradise->write_bank[3] = (svga->gdcreg[9] << 12) + ((svga->gdcreg[6] & 0x08) ? 0 : 0x8000);
-        } else {
-            paradise->read_bank[0] = paradise->write_bank[0] = svga->gdcreg[0x0a] << 12;
-            paradise->read_bank[1] = paradise->write_bank[1] = (svga->gdcreg[0xa] << 12) + ((svga->gdcreg[6] & 0x08) ? 0 : 0x8000);
-            paradise->read_bank[2] = paradise->write_bank[2] = svga->gdcreg[9] << 12;
-            paradise->read_bank[3] = paradise->write_bank[3] = (svga->gdcreg[9] << 12) + ((svga->gdcreg[6] & 0x08) ? 0 : 0x8000);
-        }
-    } else {
-        paradise->read_bank[0] = paradise->read_bank[2] = svga->gdcreg[9] << 12;
-        paradise->read_bank[1] = paradise->read_bank[3] = (svga->gdcreg[9] << 12) + ((svga->gdcreg[6] & 0x08) ? 0 : 0x8000);
-        paradise->write_bank[0] = paradise->write_bank[2] = svga->gdcreg[9] << 12;
-        paradise->write_bank[1] = paradise->write_bank[3] = (svga->gdcreg[9] << 12) + ((svga->gdcreg[6] & 0x08) ? 0 : 0x8000);
-    }
+        paradise->read_bank[0] = svga->gdcreg[9] << 12;
+        paradise->read_bank[1] = paradise->read_bank[0] + 0x8000;
 
+        paradise->write_bank[0] = svga->gdcreg[0x0a] << 12;
+        paradise->write_bank[1] = paradise->write_bank[0] + 0x8000;
+
+        if ((svga->gdcreg[6] & 0x0c) == 0x00) {
+            paradise->read_bank[2] = paradise->read_bank[1] + 0x8000;
+            paradise->read_bank[3] = paradise->read_bank[2] + 0x8000;
+
+            paradise->write_bank[2] = paradise->write_bank[1] + 0x8000;
+            paradise->write_bank[3] = paradise->write_bank[2] + 0x8000;
+        } else {
+            if (svga->gdcreg[6] & 0x08) {
+                paradise->read_bank[1] = paradise->read_bank[0];
+
+                paradise->write_bank[1] = paradise->write_bank[0];
+            }
+
+            paradise->read_bank[2] = paradise->read_bank[0];
+            paradise->read_bank[3] = paradise->read_bank[1];
+
+            paradise->write_bank[2] = paradise->write_bank[0];
+            paradise->write_bank[3] = paradise->write_bank[1];
+        }
+    } else if (svga->gdcreg[0x0b] & 0x08) {
+        if ((svga->gdcreg[6] & 0x0c) == 0x00) {
+            paradise->read_bank[0] = svga->gdcreg[0x0a] << 12;
+            paradise->read_bank[1] = paradise->read_bank[0] + 0x8000;
+            paradise->read_bank[2] = svga->gdcreg[9] << 12;
+            paradise->read_bank[3] = paradise->read_bank[2] + 0x8000;
+
+            paradise->write_bank[0] = svga->gdcreg[0x0a] << 12;
+            paradise->write_bank[1] = paradise->write_bank[0] + 0x8000;
+            paradise->write_bank[2] = svga->gdcreg[9] << 12;
+            paradise->write_bank[3] = paradise->write_bank[2] + 0x8000;
+       } else if ((svga->gdcreg[6] & 0x0c) == 0x04) {
+            paradise->read_bank[0] = svga->gdcreg[0x0a] << 12;
+            paradise->read_bank[1] = (svga->gdcreg[9] << 12) + 0x8000;
+            paradise->read_bank[2] = paradise->read_bank[0];
+            paradise->read_bank[3] = paradise->read_bank[1];
+
+            paradise->write_bank[0] = svga->gdcreg[0x0a] << 12;
+            paradise->write_bank[1] = (svga->gdcreg[9] << 12) + 0x8000;
+            paradise->write_bank[2] = paradise->write_bank[0];
+            paradise->write_bank[3] = paradise->write_bank[1];
+       } else {
+            paradise->read_bank[0] = svga->gdcreg[0x0a] << 12;
+            paradise->read_bank[1] = paradise->read_bank[0];
+            paradise->read_bank[2] = paradise->read_bank[0];
+            paradise->read_bank[3] = paradise->read_bank[0];
+
+            paradise->write_bank[0] = svga->gdcreg[0x0a] << 12;
+            paradise->write_bank[1] = paradise->write_bank[0];
+            paradise->write_bank[2] = paradise->write_bank[0];
+            paradise->write_bank[3] = paradise->write_bank[0];
+       }
+    } else {
+        paradise->read_bank[0] = svga->gdcreg[9] << 12;
+        paradise->read_bank[1] = paradise->read_bank[0] + 0x8000;
+
+        paradise->write_bank[0] = svga->gdcreg[9] << 12;
+        paradise->write_bank[1] = paradise->write_bank[0] + 0x8000;
+
+        if ((svga->gdcreg[6] & 0x0c) == 0x00) {
+            paradise->read_bank[2] = paradise->read_bank[1] + 0x8000;
+            paradise->read_bank[3] = paradise->read_bank[2] + 0x8000;
+
+            paradise->write_bank[2] = paradise->write_bank[1] + 0x8000;
+            paradise->write_bank[3] = paradise->write_bank[2] + 0x8000;
+        } else {
+            if (svga->gdcreg[6] & 0x08) {
+                paradise->read_bank[1] = paradise->read_bank[0];
+
+                paradise->write_bank[1] = paradise->write_bank[0];
+            }
+
+            paradise->read_bank[2] = paradise->read_bank[0];
+            paradise->read_bank[3] = paradise->read_bank[1];
+
+            paradise->write_bank[2] = paradise->write_bank[0];
+            paradise->write_bank[3] = paradise->write_bank[1];
+        }
+   }
+
+    /* There are separate drivers for 1M and 512K/256K versions of the PVGA chips. */
     if ((svga->gdcreg[0x0b] & 0xc0) < 0xc0) {
         paradise->read_bank[1] &= 0x7ffff;
         paradise->write_bank[1] &= 0x7ffff;
@@ -291,10 +406,89 @@ paradise_remap(paradise_t *paradise)
     }
 }
 
+void
+paradise_render_4bpp_word_highres(svga_t *svga)
+{
+    int       x;
+    int       oddeven;
+    uint32_t  addr;
+    uint32_t *p;
+    uint8_t   edat[4];
+    uint8_t   dat;
+    uint32_t  changed_addr;
+
+    if ((svga->displine + svga->y_add) < 0)
+        return;
+
+    changed_addr = ((svga->memaddr & 0x3fffc) << 1);
+
+    if (svga->changedvram[changed_addr >> 12] || svga->changedvram[(changed_addr >> 12) + 1] || svga->fullchange) {
+        p = &svga->monitor->target_buffer->line[svga->displine + svga->y_add][svga->x_add];
+
+        if (svga->firstline_draw == 2000)
+            svga->firstline_draw = svga->displine;
+        svga->lastline_draw = svga->displine;
+
+        for (x = 0; x <= (svga->hdisp + svga->scrollcache); x += 8) {
+            addr    = ((svga->memaddr & 0x3fffc) << 1);
+            oddeven = 0;
+
+            oddeven = (svga->memaddr & 2) ? 1 : 0;
+            *(uint32_t *) (&edat[0]) = *(uint32_t *) (&svga->vram[addr | oddeven]) & 0x00ff00ff;
+            svga->memaddr = (svga->memaddr + 2) & svga->vram_mask;
+
+            dat  = edatlookup[edat[0] >> 6][edat[1] >> 6] | (edatlookup[edat[2] >> 6][edat[3] >> 6] << 2);
+            p[0] = svga->pallook[svga->egapal[(dat >> 4) & svga->plane_mask]];
+            p[1] = svga->pallook[svga->egapal[dat & svga->plane_mask]];
+            dat  = edatlookup[(edat[0] >> 4) & 3][(edat[1] >> 4) & 3] |
+                   (edatlookup[(edat[2] >> 4) & 3][(edat[3] >> 4) & 3] << 2);
+            p[2] = svga->pallook[svga->egapal[(dat >> 4) & svga->plane_mask]];
+            p[3] = svga->pallook[svga->egapal[dat & svga->plane_mask]];
+            dat  = edatlookup[(edat[0] >> 2) & 3][(edat[1] >> 2) & 3] |
+                   (edatlookup[(edat[2] >> 2) & 3][(edat[3] >> 2) & 3] << 2);
+            p[4] = svga->pallook[svga->egapal[(dat >> 4) & svga->plane_mask]];
+            p[5] = svga->pallook[svga->egapal[dat & svga->plane_mask]];
+            dat  = edatlookup[edat[0] & 3][edat[1] & 3] | (edatlookup[edat[2] & 3][edat[3] & 3] << 2);
+            p[6] = svga->pallook[svga->egapal[(dat >> 4) & svga->plane_mask]];
+            p[7] = svga->pallook[svga->egapal[dat & svga->plane_mask]];
+
+            p += 8;
+        }
+    }
+}
+
+static int
+paradise_mode_is_word(svga_t *svga)
+{
+    int func_nr;
+
+    if (svga->fb_only)
+        func_nr = 0;
+    else {
+        if (svga->force_dword_mode)
+            func_nr = VAR_DWORD_MODE;
+        else if (svga->crtc[0x14] & 0x40)
+            func_nr = svga->packed_chain4 ? VAR_BYTE_MODE : VAR_DWORD_MODE;
+        else if (svga->crtc[0x17] & 0x40)
+            func_nr = VAR_BYTE_MODE;
+        else if (svga->crtc[0x17] & 0x20)
+            func_nr = VAR_WORD_MODE_MA15;
+        else
+            func_nr = VAR_WORD_MODE_MA13;
+
+        if (!(svga->crtc[0x17] & 0x01))
+            func_nr |= VAR_ROW0_MA13;
+        if (!(svga->crtc[0x17] & 0x02))
+            func_nr |= VAR_ROW1_MA14;
+    }
+
+    return (func_nr == 2);
+}
+
 void
 paradise_recalctimings(svga_t *svga)
 {
-    const paradise_t *paradise = (paradise_t *) svga->priv;
+    paradise_t *paradise = (paradise_t *) svga->priv;
 
     svga->lowres = !(svga->gdcreg[0x0e] & 0x01);
 
@@ -309,17 +503,30 @@ paradise_recalctimings(svga_t *svga)
             svga->vblankstart |= 0x400;
         if (svga->crtc[0x3e] & 0x10)
             svga->split |= 0x400;
-
-        svga->interlace = !!(svga->crtc[0x2d] & 0x20);
     }
 
+    if (paradise->type >= WD90C11)
+        svga->interlace = !!(svga->crtc[0x2d] & 0x20);
+
     if (paradise->type < WD90C30) {
         if ((svga->gdcreg[6] & 1) || (svga->attrregs[0x10] & 1)) {
             if ((svga->bpp >= 8) && !svga->lowres) {
                 svga->render = svga_render_8bpp_highres;
-                svga->vram_display_mask = (svga->crtc[0x2f] & 0x02) ? 0x3ffff : paradise->vram_mask;
+                if (paradise->type < WD90C11)
+                    svga->vram_display_mask = (svga->crtc[0x2f] & 0x02) ? 0x3ffff : paradise->vram_mask;
             }
         }
+        if (paradise->type >= WD90C11)  switch (svga->crtc[0x2f] & 0x60) {
+            case 0x60: case 0x40:
+                svga->vram_display_mask = 0x3ffff;
+                break;
+            case 0x20:
+                svga->vram_display_mask = 0x7ffff;
+                break;
+            case 0x00:
+                svga->vram_display_mask = 0xfffff;
+                break;
+        }
     } else {
         if ((svga->gdcreg[6] & 1) || (svga->attrregs[0x10] & 1)) {
             if ((svga->bpp >= 8) && !svga->lowres) {
@@ -328,15 +535,11 @@ paradise_recalctimings(svga_t *svga)
                     svga->hdisp >>= 1;
                     if (svga->hdisp == 788)
                         svga->hdisp += 12;
-                    if (svga->hdisp == 800)
-                        svga->ma_latch -= 3;
                 } else if (svga->bpp == 15) {
                     svga->render = svga_render_15bpp_highres;
                     svga->hdisp >>= 1;
                     if (svga->hdisp == 788)
                         svga->hdisp += 12;
-                    if (svga->hdisp == 800)
-                        svga->ma_latch -= 3;
                 } else
                     svga->render = svga_render_8bpp_highres;
 
@@ -347,6 +550,51 @@ paradise_recalctimings(svga_t *svga)
                                        a windowed DOS box in Win3.x*/
         }
     }
+
+    /*
+       Yes, this is basically hack but I'm going to look at a proper rewrite in
+       86Box 6.0.
+     */
+    if ((paradise->type == WD90C11) && (svga->hdisp == 1024) &&
+        (svga->render == svga_render_4bpp_highres) && paradise_mode_is_word(svga))
+        svga->render = paradise_render_4bpp_word_highres;
+}
+
+uint32_t
+paradise_decode_addr(paradise_t *paradise, uint32_t addr, int write)
+{
+    svga_t   *svga            = &paradise->svga;
+    int       memory_map_mode = (svga->gdcreg[6] >> 2) & 3;
+
+    addr &= 0x1ffff;
+
+    switch (memory_map_mode) {
+        case 0:
+            break;
+        case 1:
+            if (addr >= 0x10000)
+                return 0xffffffff;
+            break;
+        case 2:
+            addr -= 0x10000;
+            if (addr >= 0x8000)
+                return 0xffffffff;
+            break;
+        default:
+        case 3:
+            addr -= 0x18000;
+            if (addr >= 0x8000)
+                return 0xffffffff;
+            break;
+    }
+
+    if (write)
+        addr = (addr & 0x7fff) + paradise->write_bank[(addr >> 15) & 3];
+    else
+        addr = (addr & 0x7fff) + paradise->read_bank[(addr >> 15) & 3];
+
+
+    return addr;
 }
 
 static void
@@ -357,7 +605,9 @@ paradise_write(uint32_t addr, uint8_t val, void *priv)
     uint32_t    prev_addr;
     uint32_t    prev_addr2;
 
-    addr = (addr & 0x7fff) + paradise->write_bank[(addr >> 15) & 3];
+    addr = paradise_decode_addr(paradise, addr, 1);
+    if (addr == 0xffffffff)
+        return;
 
     /*Could be done in a better way but it works.*/
     if (!svga->lowres || (svga->attrregs[0x10] & 0x40)) {
@@ -390,7 +640,9 @@ paradise_writew(uint32_t addr, uint16_t val, void *priv)
     uint32_t    prev_addr;
     uint32_t    prev_addr2;
 
-    addr = (addr & 0x7fff) + paradise->write_bank[(addr >> 15) & 3];
+    addr = paradise_decode_addr(paradise, addr, 1);
+    if (addr == 0xffffffff)
+        return;
 
     /*Could be done in a better way but it works.*/
     if (!svga->lowres || (svga->attrregs[0x10] & 0x40)) {
@@ -423,7 +675,9 @@ paradise_read(uint32_t addr, void *priv)
     uint32_t    prev_addr;
     uint32_t    prev_addr2;
 
-    addr = (addr & 0x7fff) + paradise->read_bank[(addr >> 15) & 3];
+    addr = paradise_decode_addr(paradise, addr, 0);
+    if (addr == 0xffffffff)
+        return 0xff;
 
     /*Could be done in a better way but it works.*/
     if (!svga->lowres || (svga->attrregs[0x10] & 0x40)) {
@@ -456,7 +710,9 @@ paradise_readw(uint32_t addr, void *priv)
     uint32_t    prev_addr;
     uint32_t    prev_addr2;
 
-    addr = (addr & 0x7fff) + paradise->read_bank[(addr >> 15) & 3];
+    addr = paradise_decode_addr(paradise, addr, 0);
+    if (addr == 0xffffffff)
+        return 0xffff;
 
     /*Could be done in a better way but it works.*/
     if (!svga->lowres || (svga->attrregs[0x10] & 0x40)) {
@@ -482,7 +738,7 @@ paradise_readw(uint32_t addr, void *priv)
 }
 
 void *
-paradise_init(const device_t *info, uint32_t memsize)
+paradise_init(const device_t *info, uint32_t memory)
 {
     paradise_t *paradise = malloc(sizeof(paradise_t));
     svga_t     *svga     = &paradise->svga;
@@ -493,35 +749,35 @@ paradise_init(const device_t *info, uint32_t memsize)
     else
         video_inform(VIDEO_FLAG_TYPE_SPECIAL, &timing_paradise_wd90c);
 
-    paradise->memory = memsize >> 10;
+    paradise->memory = memory;
 
     switch (info->local) {
         case PVGA1A:
-            svga_init(info, svga, paradise, memsize, /*256kb*/
+            svga_init(info, svga, paradise, (memory << 10), /*256kb default*/
                       paradise_recalctimings,
                       paradise_in, paradise_out,
                       NULL,
                       NULL);
-            paradise->vram_mask = memsize - 1;
-            svga->decode_mask   = memsize - 1;
+            paradise->vram_mask = (memory << 10) - 1;
+            svga->decode_mask   = (memory << 10) - 1;
             break;
         case WD90C11:
-            svga_init(info, svga, paradise, 1 << 19, /*512kb*/
+            svga_init(info, svga, paradise, (memory << 10), /*512kb default*/
                       paradise_recalctimings,
                       paradise_in, paradise_out,
                       NULL,
                       NULL);
-            paradise->vram_mask = (1 << 19) - 1;
-            svga->decode_mask   = (1 << 19) - 1;
+            paradise->vram_mask = (memory << 10) - 1;
+            svga->decode_mask   = (memory << 10) - 1;
             break;
         case WD90C30:
-            svga_init(info, svga, paradise, memsize,
+            svga_init(info, svga, paradise, (memory << 10),
                       paradise_recalctimings,
                       paradise_in, paradise_out,
                       NULL,
                       NULL);
-            paradise->vram_mask = memsize - 1;
-            svga->decode_mask   = memsize - 1;
+            paradise->vram_mask = (memory << 10) - 1;
+            svga->decode_mask   = (memory << 10) - 1;
             svga->ramdac        = device_add(&sc11487_ramdac_device); /*Actually a Winbond W82c487-80, probably a clone.*/
             break;
 
@@ -529,6 +785,12 @@ paradise_init(const device_t *info, uint32_t memsize)
             break;
     }
 
+    svga->read = paradise_read;
+    svga->readw = paradise_readw;
+    svga->readl = NULL;
+    svga->write = paradise_write;
+    svga->writew = paradise_writew;
+    svga->writel = NULL;
     mem_mapping_set_handler(&svga->mapping, paradise_read, paradise_readw, NULL, paradise_write, paradise_writew, NULL);
     mem_mapping_set_p(&svga->mapping, paradise);
 
@@ -560,13 +822,15 @@ paradise_init(const device_t *info, uint32_t memsize)
 
     paradise->type = info->local;
 
+    svga->hoverride = 1;
+
     return paradise;
 }
 
 static void *
 paradise_pvga1a_ncr3302_init(const device_t *info)
 {
-    paradise_t *paradise = paradise_init(info, 1 << 18);
+    paradise_t *paradise = paradise_init(info, 256);
 
     if (paradise)
         rom_init(&paradise->bios_rom, "roms/machines/3302/c000-wd_1987-1989-740011-003058-019c.bin", 0xc0000, 0x8000, 0x7fff, 0, MEM_MAPPING_EXTERNAL);
@@ -577,7 +841,7 @@ paradise_pvga1a_ncr3302_init(const device_t *info)
 static void *
 paradise_pvga1a_pc2086_init(const device_t *info)
 {
-    paradise_t *paradise = paradise_init(info, 1 << 18);
+    paradise_t *paradise = paradise_init(info, 256);
 
     if (paradise)
         rom_init(&paradise->bios_rom, "roms/machines/pc2086/40186.ic171", 0xc0000, 0x8000, 0x7fff, 0, MEM_MAPPING_EXTERNAL);
@@ -588,7 +852,7 @@ paradise_pvga1a_pc2086_init(const device_t *info)
 static void *
 paradise_pvga1a_pc3086_init(const device_t *info)
 {
-    paradise_t *paradise = paradise_init(info, 1 << 18);
+    paradise_t *paradise = paradise_init(info, 256);
 
     if (paradise)
         rom_init(&paradise->bios_rom, "roms/machines/pc3086/c000.bin", 0xc0000, 0x8000, 0x7fff, 0, MEM_MAPPING_EXTERNAL);
@@ -600,12 +864,9 @@ static void *
 paradise_pvga1a_standalone_init(const device_t *info)
 {
     paradise_t *paradise;
-    uint32_t    memory = 512;
+    uint32_t memsize = device_get_config_int("memory");
 
-    memory = device_get_config_int("memory");
-    memory <<= 10;
-
-    paradise = paradise_init(info, memory);
+    paradise = paradise_init(info, memsize);
 
     if (paradise)
         rom_init(&paradise->bios_rom, "roms/video/pvga1a/BIOS.BIN", 0xc0000, 0x8000, 0x7fff, 0, MEM_MAPPING_EXTERNAL);
@@ -622,7 +883,7 @@ paradise_pvga1a_standalone_available(void)
 static void *
 paradise_wd90c11_megapc_init(const device_t *info)
 {
-    paradise_t *paradise = paradise_init(info, 0);
+    paradise_t *paradise = paradise_init(info, 512);
 
     if (paradise)
         rom_init_interleaved(&paradise->bios_rom,
@@ -636,7 +897,7 @@ paradise_wd90c11_megapc_init(const device_t *info)
 static void *
 paradise_wd90c11_standalone_init(const device_t *info)
 {
-    paradise_t *paradise = paradise_init(info, 0);
+    paradise_t *paradise = paradise_init(info, 512);
 
     if (paradise)
         rom_init(&paradise->bios_rom, "roms/video/wd90c11/WD90C11.VBI", 0xc0000, 0x8000, 0x7fff, 0, MEM_MAPPING_EXTERNAL);
@@ -654,12 +915,9 @@ static void *
 paradise_wd90c30_standalone_init(const device_t *info)
 {
     paradise_t *paradise;
-    uint32_t    memory = 512;
+    uint32_t memsize = device_get_config_int("memory");
 
-    memory = device_get_config_int("memory");
-    memory <<= 10;
-
-    paradise = paradise_init(info, memory);
+    paradise = paradise_init(info, memsize);
 
     if (paradise)
         rom_init(&paradise->bios_rom, "roms/video/wd90c30/90C30-LR.VBI", 0xc0000, 0x8000, 0x7fff, 0, MEM_MAPPING_EXTERNAL);
@@ -707,7 +965,7 @@ const device_t paradise_pvga1a_pc2086_device = {
     .init          = paradise_pvga1a_pc2086_init,
     .close         = paradise_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = paradise_speed_changed,
     .force_redraw  = paradise_force_redraw,
     .config        = NULL
@@ -721,7 +979,7 @@ const device_t paradise_pvga1a_pc3086_device = {
     .init          = paradise_pvga1a_pc3086_init,
     .close         = paradise_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = paradise_speed_changed,
     .force_redraw  = paradise_force_redraw,
     .config        = NULL
@@ -730,27 +988,21 @@ const device_t paradise_pvga1a_pc3086_device = {
 static const device_config_t paradise_pvga1a_config[] = {
   // clang-format off
     {
-        .name = "memory",
-        .description = "Memory size",
-        .type = CONFIG_SELECTION,
-        .default_int = 512,
-        .selection = {
-            {
-                .description = "256 kB",
-                .value = 256
-            },
-            {
-                .description = "512 kB",
-                .value = 512
-            },
-            {
-                .description = ""
-            }
-        }
+        .name           = "memory",
+        .description    = "Memory size",
+        .type           = CONFIG_SELECTION,
+        .default_string = NULL,
+        .default_int    = 512,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = {
+            { .description = "256 KB", .value = 256 },
+            { .description = "512 KB", .value = 512 },
+            { .description = ""                     }
+        },
+        .bios           = { { 0 } }
     },
-    {
-        .type = CONFIG_END
-    }
+    { .name = "", .description = "", .type = CONFIG_END }
   // clang-format on
 };
 
@@ -762,7 +1014,7 @@ const device_t paradise_pvga1a_ncr3302_device = {
     .init          = paradise_pvga1a_ncr3302_init,
     .close         = paradise_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = paradise_speed_changed,
     .force_redraw  = paradise_force_redraw,
     .config        = paradise_pvga1a_config
@@ -776,7 +1028,7 @@ const device_t paradise_pvga1a_device = {
     .init          = paradise_pvga1a_standalone_init,
     .close         = paradise_close,
     .reset         = NULL,
-    { .available = paradise_pvga1a_standalone_available },
+    .available     = paradise_pvga1a_standalone_available,
     .speed_changed = paradise_speed_changed,
     .force_redraw  = paradise_force_redraw,
     .config        = paradise_pvga1a_config
@@ -790,7 +1042,7 @@ const device_t paradise_wd90c11_megapc_device = {
     .init          = paradise_wd90c11_megapc_init,
     .close         = paradise_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = paradise_speed_changed,
     .force_redraw  = paradise_force_redraw,
     .config        = NULL
@@ -804,7 +1056,7 @@ const device_t paradise_wd90c11_device = {
     .init          = paradise_wd90c11_standalone_init,
     .close         = paradise_close,
     .reset         = NULL,
-    { .available = paradise_wd90c11_standalone_available },
+    .available     = paradise_wd90c11_standalone_available,
     .speed_changed = paradise_speed_changed,
     .force_redraw  = paradise_force_redraw,
     .config        = NULL
@@ -813,27 +1065,22 @@ const device_t paradise_wd90c11_device = {
 static const device_config_t paradise_wd90c30_config[] = {
   // clang-format off
     {
-        .name = "memory",
-        .description = "Memory size",
-        .type = CONFIG_SELECTION,
-        .default_int = 1024,
-        .selection = {
-            {
-                .description = "512 kB",
-                .value = 512
-            },
-            {
-                .description = "1 MB",
-                .value = 1024
-            },
-            {
-                .description = ""
-            }
-        }
+        .name           = "memory",
+        .description    = "Memory size",
+        .type           = CONFIG_SELECTION,
+        .default_string = NULL,
+        .default_int    = 1024,
+        .file_filter    = NULL,
+        .spinner        = { 0 },
+        .selection      = {
+            { .description = "256 KB", .value =  256 },
+            { .description = "512 KB", .value =  512 },
+            { .description = "1 MB",   .value = 1024 },
+            { .description = ""                      }
+        },
+        .bios           = { { 0 } }
     },
-    {
-        .type = CONFIG_END
-    }
+    { .name = "", .description = "", .type = CONFIG_END }
   // clang-format on
 };
 
@@ -845,7 +1092,7 @@ const device_t paradise_wd90c30_device = {
     .init          = paradise_wd90c30_standalone_init,
     .close         = paradise_close,
     .reset         = NULL,
-    { .available = paradise_wd90c30_standalone_available },
+    .available     = paradise_wd90c30_standalone_available,
     .speed_changed = paradise_speed_changed,
     .force_redraw  = paradise_force_redraw,
     .config        = paradise_wd90c30_config
